/*
 * Copyright (C) 2004 Zaheer Abbas Merali <zaheerabbas at merali.org>
 *                    Mahmood Rehemtulla  <mhr at blueyonder.co.uk>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <gtk/gtk.h>
#include <gst/gst.h>
#include <glade/glade.h>

#include <stdio.h>
#include <string.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#include "gststuff.h"
#include "gui.h"
#include "console.h"
#include "parser.h"

#ifndef GST_06
#include <gst/xoverlay/xoverlay.h>
void xvimagesink_have_size(GObject* object, gint arg0, gint arg1, gpointer user_data);
gboolean embed_xvideosink(struct gstsave_gui* gsgui);

/* inherited identity that has a signal for new media */
G_BEGIN_DECLS

#define GST_TYPE_NEWMEDIA \
  (gst_newmedia_get_type())
#define GST_NEWMEDIA(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_NEWMEDIA,GstNewmedia))
#define GST_NEWMEDIA_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_NEWMEDIA,GstNewmediaClass))
#define GST_IS_NEWMEDIA(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_NEWMEDIA))
#define GST_IS_NEWMEDIA_CLASS(obj) \
  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_NEWMEDIA))

typedef struct _GstNewmedia GstNewmedia;
typedef struct _GstNewmediaClass GstNewmediaClass;

struct _GstNewmedia {
  GstElement     element;

  GstPad        *sinkpad;
  GstPad        *srcpad;

  gboolean       triggered;
};

struct _GstNewmediaClass {
  GstElementClass parent_class;

};

GType gst_newmedia_get_type(void);

G_END_DECLS


GST_DEBUG_CATEGORY_STATIC (gst_newmedia_debug);
#define GST_CAT_DEFAULT gst_newmedia_debug

GstElementDetails gst_newmedia_details = GST_ELEMENT_DETAILS ("newmedia",
    "Generic",
    "Pass data without modification but allow app-triggered newmedia events",
    "Zaheer Abbas Merali");

#define _do_init(bla) \
    GST_DEBUG_CATEGORY_INIT (gst_newmedia_debug, "newmedia", 0, "newmedia element");

GST_BOILERPLATE_FULL (GstNewmedia, gst_newmedia, GstElement, GST_TYPE_ELEMENT,
    _do_init);

static void gst_newmedia_chain (GstPad * pad, GstData * _data);
static void gst_newmedia_trigger(GstNewmedia* newmedia);

static void
gst_newmedia_base_init (gpointer g_class)
{
  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);

  gst_element_class_set_details (gstelement_class, &gst_newmedia_details);
}


static void
gst_newmedia_class_init (GstNewmediaClass * klass)
{
  GObjectClass *gobject_class;
  GstElementClass *gstelement_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gstelement_class = GST_ELEMENT_CLASS (klass);

}

static void
gst_newmedia_init (GstNewmedia * newmedia)
{
  newmedia->sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
  gst_element_add_pad (GST_ELEMENT (newmedia), newmedia->sinkpad);
  gst_pad_set_chain_function (newmedia->sinkpad,
      GST_DEBUG_FUNCPTR (gst_newmedia_chain));
  gst_pad_set_link_function (newmedia->sinkpad, gst_pad_proxy_pad_link);
  gst_pad_set_getcaps_function (newmedia->sinkpad, gst_pad_proxy_getcaps);

  newmedia->srcpad = gst_pad_new ("src", GST_PAD_SRC);
  gst_element_add_pad (GST_ELEMENT (newmedia), newmedia->srcpad);
  gst_pad_set_link_function (newmedia->srcpad, gst_pad_proxy_pad_link);
  gst_pad_set_getcaps_function (newmedia->srcpad, gst_pad_proxy_getcaps);

  newmedia->triggered = FALSE;

  GST_FLAG_SET (newmedia, GST_ELEMENT_EVENT_AWARE);
}

static void
gst_newmedia_chain (GstPad * pad, GstData * _data)
{
  GstBuffer *buf = GST_BUFFER (_data);
  GstNewmedia *newmedia;
  GstEvent* newmedia_event;

  g_return_if_fail (pad != NULL);
  g_return_if_fail (GST_IS_PAD (pad));
  g_return_if_fail (buf != NULL);

  newmedia = GST_NEWMEDIA (gst_pad_get_parent (pad));

  if (GST_IS_EVENT (buf)) {
    GstEvent *event = GST_EVENT (buf);

    gst_pad_event_default (pad, event);
    return;
  }
  else {
    if (newmedia->triggered) {
       /* signal the new media discont */
       newmedia_event = gst_event_new_discontinuous(TRUE, GST_FORMAT_TIME, (gint64)0, GST_FORMAT_UNDEFINED);
       newmedia->triggered = FALSE;
       gst_pad_push (newmedia->srcpad, GST_DATA(newmedia_event));

    }
    gst_pad_push (newmedia->srcpad, GST_DATA (buf));
  }
}

static void
gst_newmedia_trigger(GstNewmedia* newmedia)
{
  g_print("Triggered newmedia\n");
  newmedia->triggered = TRUE;
}

#endif



#ifdef GNOME
#include <gst/gconf/gconf.h>
#include <profiles/gnome-media-profiles.h>
#include <gtk/gtk.h>
#include <gnome.h>
GnomeProgram* program;
#endif


#include "gtkvumeter.h"


/* ugly  for vorbis bitrates less than 48k/sec */
#define BITRATE_MAP_NUMBITRATES 2
int bitrate_map[2][BITRATE_MAP_NUMBITRATES]= { {16, 11025}, {20, 20050} };

static void
newfile_signal (GstElement* element, struct stream_details* sd)
{
  sd->curcounter++;
 
  if (element == sd->pe->sink2) {
    g_free(sd->webcaststream_fname);
    
    sd->webcaststream_fname=g_strdup_printf("%s/%s_%s_1_%d.%s", sd->dirname,
					    sd->str,sd->datestr,sd->curcounter,
					    g_strcasecmp(sd->webcasttype,"ogg")==0 ? "ogg" : "mp3");
    //printf("New Broadcast Filename %s \n",sd->webcaststream_fname);
    g_object_set(G_OBJECT(element),"location",sd->webcaststream_fname,NULL);
  }
  else if (element == sd->pe->sink1) {
    gchar* extension;
    g_free (sd->recordstream_fname);
    if (g_strcasecmp (sd->recordtype, "ogg") == 0) extension = g_strdup("ogg");
    else if (g_strcasecmp (sd->recordtype, "mp3") == 0) extension = g_strdup("mp3");
    else {
      // Gnome media profile
      GList* temp_profile = sd->recording_gm_profiles;
      while (temp_profile != NULL) {
	const gchar* tempprof = gm_audio_profile_get_id (GM_AUDIO_PROFILE(temp_profile->data));
	gchar* temp_pipeline;
	if (g_strcasecmp (tempprof, sd->recordtype) == 0) {
	  extension = g_strdup(gm_audio_profile_get_extension (GM_AUDIO_PROFILE(temp_profile->data)));
	  break;
	}
	temp_profile = temp_profile->next;
      }

    }
    sd->recordstream_fname = g_strdup_printf("%s/%s_%s_0_%d.%s", sd->dirname,
                                             sd->str, sd->datestr,
					     sd->curcounter, extension);
    g_free (extension);
    g_object_set(G_OBJECT(element),"location",sd->recordstream_fname,NULL);
  }
  else if (element == sd->pe->sink3) {
    g_free(sd->wavstream_fname);
    sd->wavstream_fname=g_strdup_printf("%s/%s_%s_2_%d.wav", sd->dirname,
					    sd->str,sd->datestr,sd->curcounter);
    g_object_set(G_OBJECT(element),"location",sd->wavstream_fname,NULL);
  }
					    

  return;
}

static void
level_callback (GstElement * element, gdouble time, gint channel,
    gdouble rms, gdouble peak, gdouble decay, gpointer user_data)
{
  gchar *label;
  gchar* tempkey;
  GtkWidget* tempwidget;
  struct gstsave_gui* gsgui=(struct gstsave_gui*)user_data;


  if (gsgui->sd->gui) {
	//tempwidget=GTK_WIDGET(g_hash_table_lookup(gsgui->table,"elapsedtime_label"));
	gdk_threads_enter();
	tempwidget=glade_xml_get_widget(gsgui->windowdefs,"elapsedtime_label");
	label=g_strdup_printf("% 07.3f",time);
	if (strlen(label)>6) label[6]='\0';
	gtk_label_set(GTK_LABEL(tempwidget), label);
	g_free(label);
	tempkey=g_strdup_printf("volume%d_vumeter",channel);
	tempwidget=GTK_WIDGET(g_hash_table_lookup(gsgui->table,tempkey));
	g_free(tempkey);
	gtk_vumeter_set_levels (GTK_VUMETER(tempwidget), (gint)(rms*10.0), (gint)(peak*10.0));
	gdk_threads_leave();

  }	
  label = g_strdup_printf ("%d %.3f %.3f %.3f %.3f\n", channel, time,rms,peak,decay);
  if (gsgui->sd->ioc && gsgui->sd->ioc->is_writeable) {
  	g_io_channel_write_chars(gsgui->sd->ioc,label,-1,NULL,NULL);
  	g_io_channel_flush(gsgui->sd->ioc,NULL);
  }
  if (gsgui->sd->consolelevel==TRUE) {
     g_print(label);
     gsgui->sd->consolelevel=FALSE;
  }
  g_free (label);
  
}

GstElement *
element_create (char *name, char *element)
  /*
   * create the element
   * print an error if it can't be created
   * return NULL if it couldn't be created
   * return element if it did work
   */
{
  GstElement *el = NULL;

  el = (GstElement *) gst_element_factory_make (element, name);
  if (el == NULL)
  {
    fprintf (stderr, "Could not create element %s (%s) !\n", name, element);
    return NULL;
  }
  else
    return el;
}

static void remove_webcast_encoder(struct stream_details* sd)
{
  /*lets remove the elements and unlink the pads*/
  if (sd->pe->tempas1) {
    gst_element_unlink_many (sd->pe->tempac1, sd->pe->tempas1, NULL);
    if (sd->pe->tempas2) {
      gst_element_unlink (sd->pe->tempas1, sd->pe->tempas2);
      if (sd->pe->enc2_ac) gst_element_unlink_many (sd->pe->tempas2, sd->pe->enc2_ac, sd->pe->enc2, NULL);
      else gst_element_unlink (sd->pe->tempas2, sd->pe->enc2);
    }
    else {
      if (sd->pe->enc2_ac) gst_element_unlink_many (sd->pe->tempac1, sd->pe->enc2_ac, sd->pe->enc2, NULL);
      else gst_element_unlink_many (sd->pe->tempac1, sd->pe->enc2, NULL);
    }
  }
  else {
    if (sd->pe->enc2_ac) gst_element_unlink_many (sd->pe->tempac1, sd->pe->enc2_ac, sd->pe->enc2, NULL);
    else gst_element_unlink (sd->pe->tempac1, sd->pe->enc2);
  }
  if (sd->pe->enc2_mux)
    gst_element_unlink_many (sd->pe->enc2, sd->pe->enc2_mux, sd->pe->tee2, NULL);
  else 
    gst_element_unlink_many (sd->pe->enc2, sd->pe->tee2, NULL);
  
  if (sd->sub_stream_status[1]) {
    gst_pad_disconnect (sd->pe->tee2_src1, gst_element_get_pad(sd->pe->sink2,"sink"));
    gst_element_release_request_pad(sd->pe->tee2, sd->pe->tee2_src1);
  }
  if (sd->sub_stream_status[2]) {
    gst_pad_disconnect (sd->pe->tee2_src2, gst_element_get_pad(sd->pe->sinkweb,"sink"));
    gst_element_release_request_pad(sd->pe->tee2, sd->pe->tee2_src2);
  }
  gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->tee2);
  sd->pe->tee2=NULL;
  if (sd->pe->tempas1) {
    gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->tempas1);
    sd->pe->tempas1=NULL;
  }
  if (sd->pe->tempas2) {
    gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->tempas2);
    sd->pe->tempas2=NULL;
  }
  if (sd->pe->enc2_ac) gst_bin_remove (GST_BIN(sd->pe->pipeline), sd->pe->enc2_ac);
  gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->enc2);
  if (sd->pe->enc2_mux) gst_bin_remove (GST_BIN(sd->pe->pipeline), sd->pe->enc2_mux);
  
}

static void add_webcast_encoder(struct stream_details* sd, gboolean start)
{
  GstStaticCaps tempcaps,tempcaps2, tempcaps3;
  gint i;
  gint rateneeded;
  gboolean ratedone;
  gboolean add_queue=FALSE;
  gboolean add_tee=FALSE;
  gboolean add_thread=FALSE;
  gboolean add_tempac1=FALSE;

  tempcaps.caps.type=0;
  tempcaps2.caps.type=0;

  /*if (!sd->pe->webcast_encoder_thread) {
    sd->pe->webcast_encoder_thread=gst_thread_new("webcast_encoder_thread");
    add_thread=TRUE;
  }
  if (!sd->pe->webcast_encoder_queue) { 
    sd->pe->webcast_encoder_queue=element_create("webcast_encoder_queue","queue");
    add_queue=TRUE;
    }*/
  if (!sd->pe->tee2) {
    sd->pe->tee2 = element_create ("tee2", "tee");
    add_tee=TRUE;
  }
  if (!sd->pe->tempac1) {
    sd->pe->tempac1 = element_create("tempac1","audioconvert");
    add_tempac1=TRUE;
  }
  //g_print("Required for adding: tee - %d audioconvert - %d\n", add_tee, ad_tempac1);
  /* do ugly stuff */
  rateneeded=44100;
  ratedone = FALSE;
  if (g_strcasecmp (sd->webcasttype, "mp3") == 0) {
    /* check plugin version of lame */
    GstPlugin* lame_plugin = gst_registry_pool_find_plugin ("lame");
    if (lame_plugin && 
	lame_plugin->desc.version[2] == '8' &&
	atof(lame_plugin->desc.version+4) > 3.1) {
      rateneeded=44100;
      ratedone = TRUE;
    }
  }
  if (sd->bitrate[1]<64 && !ratedone) {
    for(i=0;i<BITRATE_MAP_NUMBITRATES;i++) {
      if (sd->bitrate[1]>=bitrate_map[i][0]) {
	rateneeded=bitrate_map[i][1];
      }
    }
    switch (rateneeded) {
    case 11025:
      sd->pe->tempas1=element_create("wcas1","audioscale");
      sd->pe->tempas2=element_create("wcas2","audioscale");
      break;
    case 22050:
      sd->pe->tempas1=element_create("wcas1","audioscale");
      sd->pe->tempas2=NULL;
      break;
    }
  }

  if (g_strcasecmp(sd->webcasttype,"ogg")==0) {
    sd->pe->enc2_ac = element_create ("ac_enc2", "audioconvert"); 
    
    sd->pe->enc2 = element_create("enc2","rawvorbisenc");
    g_object_set (G_OBJECT(sd->pe->enc2), "max-bitrate",(int)(sd->bitrate[1]*1000),NULL);
    sd->pe->enc2_mux = element_create ("enc2_mux", "oggmux");
    
  }
  else {
    sd->pe->enc2 = element_create ("enc2","lame");
    g_object_set (G_OBJECT(sd->pe->enc2), "mode", sd->mode[1], NULL);
    g_object_set (G_OBJECT(sd->pe->enc2), "bitrate", sd->bitrate[1], NULL);
    g_object_set (G_OBJECT(sd->pe->enc2), "compression-ratio", 0.0, NULL);
    sd->pe->enc2_mux = NULL;
    sd->pe->enc2_ac = NULL;
  }
  //if (add_queue) gst_bin_add(GST_BIN(sd->pe->webcast_encoder_thread),sd->pe->webcast_encoder_queue);
  if (add_tempac1) gst_bin_add(GST_BIN(sd->pe->pipeline),sd->pe->tempac1);
  if (sd->pe->tempas1) gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->tempas1);
  if (sd->pe->tempas2) gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->tempas2);
  if (sd->pe->enc2_mux) gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->enc2_mux);
  if (sd->pe->enc2_ac) gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->enc2_ac);
  gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->enc2);
  if (add_tee) gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->tee2);
  //if (add_thread) gst_bin_add(GST_BIN(sd->pe->pipeline),sd->pe->webcast_encoder_thread);
  /* request one pad from tee */
  if (add_tempac1) {
    GstCaps* wcac_caps = gst_caps_from_string ("audio/x-raw-int, rate=44100, channels=2");
    sd->pe->tee1_src2 = gst_element_get_request_pad (sd->pe->tee, "src%d");
    gst_pad_link_filtered(sd->pe->tee1_src2,gst_element_get_pad(sd->pe->tempac1,"sink"),wcac_caps);
  }
  
  //gst_pad_connect(gst_element_get_pad(sd->pe->webcast_encoder_queue,"src"),gst_element_get_pad(sd->pe->tempac1,"sink"));
				   
  if (sd->pe->tempas1) {
    gchar* tempcaps_str = g_strdup_printf("audio/x-raw-int, rate=22050, channels=%d",sd->mode[1]==0 ? 2 : 1); 
    GstCaps* wcas1_caps = gst_caps_from_string(tempcaps_str);
    g_free (tempcaps_str);
    
    gst_pad_connect (gst_element_get_pad(sd->pe->tempac1,"src"), gst_element_get_pad (sd->pe->tempas1, "sink"));
    if (sd->pe->tempas2) {
      tempcaps_str = g_strdup_printf("audio/x-raw-int, rate=11025, channels=%d",sd->mode[1]==0 ?2:1);
      GstCaps* wcas2_caps = gst_caps_from_string(tempcaps_str);

      g_free (tempcaps_str);
      gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas1, "src"), gst_element_get_pad (sd->pe->tempas2, "sink"), wcas1_caps);
      gst_caps_free (wcas1_caps);
      //g_free((void*)tempcaps.string);
      //gst_pad_connect(gst_element_get_pad(sd->pe->tempas1,"src"),gst_element_get_pad(sd->pe->tempas2,"sink"));
      if (sd->pe->enc2_ac) {
        GstCaps* enc2ac_caps;
	tempcaps_str = g_strdup_printf("audio/x-raw-float, channels=%d",sd->mode[1]==0 ?2:1);
        enc2ac_caps = gst_caps_from_string (tempcaps_str);
	g_free (tempcaps_str);
	
        gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas2, "src"), gst_element_get_pad (sd->pe->enc2_ac, "sink"), wcas2_caps);
	gst_element_link_filtered (sd->pe->enc2_ac, sd->pe->enc2, enc2ac_caps);
	gst_caps_free (enc2ac_caps);
      }
      else {
        gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas2, "src"), gst_element_get_pad (sd->pe->enc2, "sink"), wcas2_caps);
      }
      gst_caps_free (wcas2_caps);
      //g_free((void*)tempcaps3.string);
    }
    else {
      if (sd->pe->enc2_ac) {
        GstCaps* enc2ac_caps;
	tempcaps_str = g_strdup_printf("audio/x-raw-float, channels=%d",sd->mode[1]==0 ?2:1);
        enc2ac_caps = gst_caps_from_string (tempcaps_str);
	g_free (tempcaps_str);
	
        gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas1, "src"), gst_element_get_pad (sd->pe->enc2_ac, "sink"), wcas1_caps);
	gst_element_link_filtered (sd->pe->enc2_ac, sd->pe->enc2, enc2ac_caps);
	gst_caps_free (enc2ac_caps);
      }
      else {

        gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas1, "src"), gst_element_get_pad (sd->pe->enc2, "sink"), wcas1_caps);
        gst_caps_free (wcas1_caps);
      }
    }
 
  }
  else {
    gchar* tempcaps_str = g_strdup_printf("audio/x-raw-int, rate=44100, channels=%d", sd->mode[1] == 0 ? 2:1);
    GstCaps* wcac_caps = gst_caps_from_string(tempcaps_str);
    GstCaps* enc2ac_caps;
    g_free (tempcaps_str);
    tempcaps_str = g_strdup_printf("audio/x-raw-float, channels=%d",
      sd->mode[1] == 0 ? 2:1);
    enc2ac_caps = gst_caps_from_string (tempcaps_str);
    if (sd->pe->enc2_ac) {
      gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempac1,"src"), 
                             gst_element_get_pad (sd->pe->enc2_ac, "sink"),
                             wcac_caps);
      gst_pad_link_filtered (gst_element_get_pad (sd->pe->enc2_ac, "src"),
                             gst_element_get_pad (sd->pe->enc2, "sink"),
			     enc2ac_caps);
    }
    else {
      gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempac1,"src"), 
                             gst_element_get_pad (sd->pe->enc2, "sink"),
                             wcac_caps);
    }
    gst_caps_free (wcac_caps);
    gst_caps_free (enc2ac_caps);
  }

  if (!start && add_tee) {
    if (sd->sub_stream_status[1]) {
      sd->pe->tee2_src1=gst_element_get_request_pad(sd->pe->tee2,"src%d");
      gst_pad_connect (sd->pe->tee2_src1, gst_element_get_pad(sd->pe->sink2,"sink"));
    }
    if (sd->sub_stream_status[2]) {
      sd->pe->tee2_src2=gst_element_get_request_pad(sd->pe->tee2,"src%d");
      gst_pad_connect (sd->pe->tee2_src2, gst_element_get_pad(sd->pe->sinkweb,"sink"));
    }
  }
  if (sd->pe->enc2_mux)
    gst_element_link_many (sd->pe->enc2, sd->pe->enc2_mux, sd->pe->tee2, NULL);
  else
    gst_element_link_many (sd->pe->enc2, sd->pe->tee2, NULL);
    
}

int stop_record_stream(struct stream_details* sd)
{

  if (sd->sub_stream_status[0]) {
    if (sd->pe->enc1_mux) gst_element_unlink_many (sd->pe->enc1, sd->pe->enc1_mux, sd->pe->sink1, NULL);
    else gst_element_unlink_many (sd->pe->enc1, sd->pe->sink1, NULL);
    gst_pad_disconnect (sd->pe->tee1_src1,
			gst_element_get_pad(sd->pe->tempac2,"sink"));
    if (sd->pe->tempas3) {
      gst_element_unlink_many (sd->pe->tempac2, sd->pe->tempas3, NULL);
      if (sd->pe->tempas4) {
        if (sd->pe->enc1_ac) gst_element_unlink_many (sd->pe->tempas3, sd->pe->tempas4, sd->pe->enc1_ac, sd->pe->enc1, NULL);
	else gst_element_unlink_many (sd->pe->tempas3, sd->pe->tempas4, sd->pe->enc1, NULL);
      }
      else {
        if (sd->pe->enc1_ac) gst_element_unlink_many (sd->pe->tempas3, sd->pe->enc1_ac, sd->pe->enc1, NULL);
	else gst_element_unlink_many (sd->pe->tempas3, sd->pe->enc1, NULL);
      }
    }
    else {
      if (sd->pe->enc1_ac) gst_element_unlink_many (sd->pe->tempac2, sd->pe->enc1_ac, sd->pe->enc1, NULL);
      else gst_element_unlink_many (sd->pe->tempac2, sd->pe->enc1, NULL);
    }
    gst_bin_remove_many (GST_BIN(sd->pe->pipeline), sd->pe->sink1, sd->pe->enc1, sd->pe->tempac2, NULL);
    if (sd->pe->enc1_ac) gst_bin_remove (GST_BIN(sd->pe->pipeline), sd->pe->enc1_ac); 
    if (sd->pe->enc1_mux) gst_bin_remove (GST_BIN(sd->pe->pipeline), sd->pe->enc1_mux);
    sd->pe->enc1_mux = NULL;
    sd->pe->enc1_ac = NULL;
    if (sd->pe->tempas3) {
      gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->tempas3);
      sd->pe->tempas3=NULL;
    }
    if (sd->pe->tempas4) {
      gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->tempas4);
      sd->pe->tempas4=NULL;
    }

    gst_element_release_request_pad(sd->pe->tee, sd->pe->tee1_src1);
    sd->sub_stream_status[0]=0;
    return(0);
  }
  return -1;
}

int stop_webcast_record_stream(struct stream_details* sd)
{
  // g_free(mp3stream1fname);
  
  if (sd->sub_stream_status[1]) {
    gst_pad_disconnect (sd->pe->tee2_src1, gst_element_get_pad(sd->pe->sink2,"sink"));
    gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->sink2);
    // free sink1
    //gst_element_destroy(sink2);
    
    gst_element_release_request_pad(sd->pe->tee2, sd->pe->tee2_src1);
    sd->sub_stream_status[1]=0;
    return(0);
  }
  return -1;
}

int stop_webcast_substream(struct stream_details* sd)
{
  if (sd->sub_stream_status[2]) {
    gst_pad_disconnect (sd->pe->tee2_src2, gst_element_get_pad(sd->pe->sinkweb,"sink"));
    gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->sinkweb);
    gst_element_release_request_pad(sd->pe->tee2, sd->pe->tee2_src2);
    sd->sub_stream_status[2]=0;
    return(0);
  }
  return -1;
}
int stop_visualisation_substream(struct stream_details* sd)
{
  /* currently bug in gst means this has to be disabled */

  if (sd->sub_stream_status[3]) {
    gst_element_unlink_many (sd->pe->queue, sd->pe->s2m, sd->pe->vis,
			     sd->pe->color, sd->pe->vsink, NULL);

    gst_pad_disconnect(sd->pe->tee1_src3, 
		       gst_element_get_pad (sd->pe->queue, "sink"));
    gst_bin_remove_many (GST_BIN (sd->pe->vis_thread), sd->pe->queue, sd->pe->s2m, sd->pe->vis, sd->pe->color, NULL);

    gst_element_set_state (sd->pe->vsink, GST_STATE_NULL);
    //gst_bin_remove (GST_BIN (sd->pe->vis_thread), sd->pe->vsink);
    //gst_bin_remove (GST_BIN (sd->pe->pipeline), sd->pe->vis_thread);
    gtk_widget_set_size_request(glade_xml_get_widget (sd->gsgui->windowdefs, "visualisation"),0,0);
    gtk_widget_hide (glade_xml_get_widget (sd->gsgui->windowdefs, "visualisation"));
			
    gst_element_release_request_pad(sd->pe->tee, sd->pe->tee1_src3);
    sd->sub_stream_status[3]=0;
    return 0;
    
  }
  
  return -1;
}

int stop_listen_substream(struct stream_details* sd)
{
  if (sd->sub_stream_status[5]) {
    gst_element_set_state(GST_ELEMENT(sd->pe->listen_thread),GST_STATE_NULL);
    gst_pad_disconnect(gst_element_get_pad(sd->pe->listen_queue,"src"),
		       gst_element_get_pad(sd->pe->audiosink,"sink"));
    
    gst_pad_disconnect(GST_PAD(sd->pe->tee1_src5), 
		       gst_element_get_pad(sd->pe->listen_queue,"sink"));
    
    gst_bin_remove_many (GST_BIN(sd->pe->listen_thread), sd->pe->listen_queue, sd->pe->audiosink, NULL);

    //gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->listen_thread);
    /* need to free the listen_thread and the removed elements */
    
    gst_element_release_request_pad(sd->pe->tee, GST_PAD(sd->pe->tee1_src5));
    
    gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->listen_thread);
    sd->sub_stream_status[5]=0;
    return(0);
  }
  return -1;
}	
int stop_wav_substream1(struct stream_details* sd)
{

  if (sd->sub_stream_status[4]) {
    gst_pad_disconnect (gst_element_get_pad(sd->pe->enc3,"src"),
			gst_element_get_pad(sd->pe->sink3,"sink"));
    gst_pad_disconnect(sd->pe->tee1_src4, gst_element_get_pad(sd->pe->enc3,"sink"));
    gst_bin_remove_many (GST_BIN(sd->pe->pipeline), sd->pe->sink3, sd->pe->enc3, NULL);
    
    gst_element_release_request_pad(sd->pe->tee, sd->pe->tee1_src4);
    sd->sub_stream_status[4]=0;
    return(0);
  }
  return -1;
}

int add_record_stream(struct stream_details* sd)
{
  GstCaps* tempcaps, *tempcaps2;
  gchar* tempcaps_str, *tempcaps2_str;
  gint rateneeded,i;
  gboolean ratedone;
  /*tempcaps.caps.type=0;
    tempcaps2.caps.type=0;*/
  rateneeded=44100;

  if (!sd->sub_stream_status[0]) {
    if (g_strcasecmp (sd->recordtype, "ogg") == 0 || g_strcasecmp (sd->recordtype, "mp3") == 0) {
      g_print ("Add & Connect %s Record Stream Bitrate:%d\n", sd->recordtype, sd->bitrate[0]);
      sd->curcounter++;
      
      sd->recordstream_fname=g_strdup_printf("%s/%s_%s_0_%d.%s",sd->dirname, sd->str,
					     sd->datestr, sd->curcounter,
					     g_strcasecmp(sd->recordtype,"ogg")==0 ? "ogg" : "mp3");
      
      if (!(sd->pe->tempac2=element_create("tempac2","audioconvert"))) return 1;
      
      /* do ugly stuff */
      /* check plugin version of lame - currently cvs of gst-plugins */
      ratedone = FALSE;
      { 
	GstPlugin* lame_plugin = gst_registry_pool_find_plugin ("lame");
	if (lame_plugin && 
	    lame_plugin->desc.version[2] == '8' &&
	    atof(lame_plugin->desc.version+4) > 3.1) {

	  rateneeded=44100;
	  ratedone = TRUE;
	}
      }
      if (sd->bitrate[0]<64 && !ratedone) {
	for(i=0;i<BITRATE_MAP_NUMBITRATES;i++) {
	  if (sd->bitrate[0]>=bitrate_map[i][0]) {
	    rateneeded=bitrate_map[i][1];
	  }
	}
	switch (rateneeded) {
	case 11025:
	  sd->pe->tempas3=element_create("ras1","audioscale");
	  sd->pe->tempas4=element_create("ras2","audioscale");
	  break;
	case 22050:
	  sd->pe->tempas3=element_create("ras1","audioscale");
	  sd->pe->tempas4=NULL;
	  break;
	case 44100:
	  sd->pe->tempas3=NULL;
	  sd->pe->tempas4=NULL;
	}
      }
      if (g_strcasecmp(sd->recordtype,"ogg")==0) {
	if (!(sd->pe->enc1 = element_create("enc1","rawvorbisenc"))) return 1;
	if (!(sd->pe->enc1_ac = element_create("enc1_ac","audioconvert"))) return 1;
	if (!(sd->pe->enc1_mux = element_create("enc1_mux","oggmux"))) return 1;
	if (sd->quality[0] > 0) {
	  g_object_set (G_OBJECT(sd->pe->enc1), "quality",(float)(sd->quality[0]/10),NULL);
	}
	else {
	  g_object_set (G_OBJECT(sd->pe->enc1), "max-bitrate",(int)(sd->bitrate[0]*1000),NULL);
	}
	
      }
      else {						
	if (!(sd->pe->enc1 = element_create ("enc1","lame"))) return 1;
	g_object_set (G_OBJECT(sd->pe->enc1), "mode", sd->mode[0], NULL);
	g_object_set (G_OBJECT(sd->pe->enc1), "compression-ratio", 0.0, NULL);
	g_object_set (G_OBJECT(sd->pe->enc1), "bitrate", sd->bitrate[0], NULL);
      }
      
      if (sd->newmedia) {
        if (!(sd->pe->sink1 = element_create ("sink1", "multifilesink"))) return 1;
	g_signal_connect (G_OBJECT(sd->pe->sink1), "newfile", G_CALLBACK(newfile_signal), sd);
      }
      else {
        if (!(sd->pe->sink1 = element_create ("sink1", "filesink"))) return 1;
      }
      g_object_set (G_OBJECT (sd->pe->sink1), "location", sd->recordstream_fname, NULL);
      /* request one pad from tee */
      sd->pe->tee1_src1 = gst_element_get_request_pad (sd->pe->tee, "src%d");
      //gst_bin_add (GST_BIN (sd->pe->record_thread), sd->pe->record_queue);
      gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->tempac2);
      if (sd->pe->tempas3) gst_bin_add(GST_BIN(sd->pe->pipeline), sd->pe->tempas3);
      if (sd->pe->tempas4) gst_bin_add(GST_BIN(sd->pe->pipeline), sd->pe->tempas4);
      gst_bin_add_many (GST_BIN (sd->pe->pipeline), sd->pe->enc1, sd->pe->sink1, NULL);
      if (sd->pe->enc1_ac) gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->enc1_ac);
      if (sd->pe->enc1_mux) gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->enc1_mux);
      tempcaps = gst_caps_from_string ("audio/x-raw-int, rate=44100, channels=2");
      gst_pad_link_filtered(sd->pe->tee1_src1,
			    gst_element_get_pad(sd->pe->tempac2,"sink"),
			    tempcaps);
      gst_caps_free(tempcaps);
      tempcaps2_str = g_strdup_printf("audio/x-raw-int, rate=%d, channels=%d",rateneeded,sd->mode[0]==0 ? 1:2);
      tempcaps2 = gst_caps_from_string (tempcaps2_str);
      g_free (tempcaps2_str);
      tempcaps2_str = g_strdup_printf("audio/x-raw-float, channels=%d",sd->mode[0]==0 ? 1:2);
      tempcaps = gst_caps_from_string (tempcaps2_str);
      g_free (tempcaps2_str);

      if (sd->pe->tempas3) {
	gst_pad_connect(gst_element_get_pad(sd->pe->tempac2,"src"),
		      gst_element_get_pad(sd->pe->tempas3,"sink"));
        if (sd->pe->tempas4) {
	  gst_pad_connect(gst_element_get_pad(sd->pe->tempas3,"src"),
	    		  gst_element_get_pad(sd->pe->tempas4,"sink"));
	  if (sd->pe->enc1_ac) {		
	    gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas4,"src"),
	  	  	           gst_element_get_pad (sd->pe->enc1_ac, "sink"),
			           tempcaps2);
            gst_pad_link_filtered (gst_element_get_pad (sd->pe->enc1_ac, "src"),
  	                           gst_element_get_pad (sd->pe->enc1, "sink"), 
				   tempcaps);

	  }
	  else {
	    gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas4,"src"),
	  	  	           gst_element_get_pad (sd->pe->enc1, "sink"),
			           tempcaps2);
          }
	
	}
        else {
	  if (sd->pe->enc1_ac) {
	    gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas3,"src"),
	  		           gst_element_get_pad (sd->pe->enc1_ac, "sink"),
			           tempcaps2);
	    gst_pad_link_filtered (gst_element_get_pad(sd->pe->enc1_ac,"src"),
	  		           gst_element_get_pad (sd->pe->enc1, "sink"),
			           tempcaps);

	  }
	  else {

	    gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempas3,"src"),
	  	  	           gst_element_get_pad (sd->pe->enc1, "sink"),
			           tempcaps2);
	  }
	  
        }
      }
      else {
        if (sd->pe->enc1_ac) {
	  gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempac2,"src"),
			         gst_element_get_pad (sd->pe->enc1_ac, "sink"),
			         tempcaps2);
	  gst_pad_link_filtered (gst_element_get_pad(sd->pe->enc1_ac,"src"),
	  		         gst_element_get_pad (sd->pe->enc1, "sink"),
			         tempcaps);
       	}	 
        else {
	  gst_pad_link_filtered (gst_element_get_pad(sd->pe->tempac2,"src"),
			         gst_element_get_pad (sd->pe->enc1, "sink"),
			         tempcaps2);
	}
      }
      gst_caps_free (tempcaps);
      gst_caps_free (tempcaps2);
      if (sd->pe->enc1_mux) {
        gst_element_connect (sd->pe->enc1, sd->pe->enc1_mux);
        gst_element_connect (sd->pe->enc1_mux, sd->pe->sink1);
      }
      else {
        gst_pad_connect (gst_element_get_pad(sd->pe->enc1,"src"), 
	  	         gst_element_get_pad(sd->pe->sink1,"sink"));
      }
      gst_element_set_state (sd->pe->tempac2, GST_STATE_READY);
      if (sd->pe->tempas3) gst_element_set_state(sd->pe->tempas3,GST_STATE_READY);
      if (sd->pe->tempas4) gst_element_set_state(sd->pe->tempas4,GST_STATE_READY);
      gst_element_set_state (sd->pe->enc1,GST_STATE_READY);
      if (sd->pe->enc1_ac) gst_element_set_state(sd->pe->enc1_ac,GST_STATE_READY);
      if (sd->pe->enc1_mux) gst_element_set_state(sd->pe->enc1_mux,GST_STATE_READY);
      gst_element_set_state(sd->pe->sink1,GST_STATE_READY);
      sd->sub_stream_status[0]=1;
      
      return 0;
    }
#ifdef GNOME
    else {
      GstElement* temp;
      GstPad* temppad;
      GList* temp_profile = sd->recording_gm_profiles;
      GError *err = NULL;
      sd->pe->enc1 = NULL;
      sd->pe->tempas4 = NULL;
      sd->pe->tempas3 = NULL;
      
      while (temp_profile != NULL) {
	const gchar* tempprof = gm_audio_profile_get_id (GM_AUDIO_PROFILE(temp_profile->data));
	gchar* temp_pipeline;
	if (g_strcasecmp (tempprof, sd->recordtype) == 0) {
	  temp_pipeline = g_strdup_printf ("(name=gmp_encoder identity name=encoder_start ! %s ! identity name=encoder_end )", gm_audio_profile_get_pipeline (GM_AUDIO_PROFILE(temp_profile->data)));
	  g_print ("pipeline is: %s\n", temp_pipeline);
	  sd->pe->enc1 = gst_parse_launch (temp_pipeline, &err);
	  g_free (temp_pipeline);
	  if (err != NULL) {
	    g_print("Error parsing gnome-media pipeline for profile: %s\n", tempprof);
	    return 1;
	  }
	  temp = gst_bin_get_by_name (GST_BIN (sd->pe->enc1), "encoder_start");
	  temppad = gst_element_get_pad (temp, "sink");
	  gst_element_add_ghost_pad (sd->pe->enc1, temppad, "sink-ghost");
	  temp = gst_bin_get_by_name (GST_BIN (sd->pe->enc1), "encoder_end");
	  temppad = gst_element_get_pad (temp, "src");
	  gst_element_add_ghost_pad (sd->pe->enc1, temppad, "src-ghost");
	  sd->pe->tempas3 = element_create("ras1","audioscale");
	  sd->pe->tempac2 = element_create("tempac2","audioconvert");
	  sd->curcounter++;
	  sd->recordstream_fname=g_strdup_printf("%s/%s_%s_0_%d.%s",sd->dirname, sd->str,
					     sd->datestr, sd->curcounter,
					     gm_audio_profile_get_extension (GM_AUDIO_PROFILE(temp_profile->data)));
	  if (!(sd->pe->sink1 = element_create ("sink1", "filesink"))) return 1;
	  g_object_set (G_OBJECT (sd->pe->sink1), "location", sd->recordstream_fname, NULL);
	  /* request one pad from tee */
	  sd->pe->tee1_src1 = gst_element_get_request_pad (sd->pe->tee, "src%d");
	  gst_bin_add_many (GST_BIN(sd->pe->pipeline), sd->pe->tempac2, sd->pe->tempas3, sd->pe->enc1, sd->pe->sink1, NULL);
	  tempcaps = gst_caps_from_string ("audio/x-raw-int, rate=44100, channels=2");
	  gst_pad_link_filtered (sd->pe->tee1_src1, gst_element_get_pad(sd->pe->tempac2, "sink"), tempcaps);
	  gst_caps_free (tempcaps);
	  gst_element_link_many (sd->pe->tempac2, sd->pe->tempas3, sd->pe->enc1, sd->pe->sink1, NULL);
	  gst_element_set_state (sd->pe->tempac2, GST_STATE_READY);
	  gst_element_set_state (sd->pe->tempas3, GST_STATE_READY);
	  gst_element_set_state (sd->pe->enc1, GST_STATE_READY);
	  gst_element_set_state (sd->pe->sink1, GST_STATE_READY);
	  sd->sub_stream_status[0] = 1;
	  return 0;
	  
	}
	temp_profile = temp_profile->next;
      }
    }

      
#endif
  }
  return -1;
}

int add_webcast_record_stream(struct stream_details* sd,gboolean start)
{
  if (!sd->sub_stream_status[1]) {
    g_print ("Add & Connect %s Webcast Quality Stream Bitrate:%d\n", sd->webcasttype, sd->bitrate[1]);
    
    if (!start) {
      remove_webcast_encoder(sd);
      add_webcast_encoder(sd,FALSE);
    }
		

    sd->curcounter++;
    sd->webcaststream_fname=g_strdup_printf("%s/%s_%s_1_%d.%s",sd->dirname, sd->str,
					    sd->datestr, sd->curcounter,
					    g_strcasecmp(sd->webcasttype,"ogg")==0 ? "ogg" : "mp3");
    if (sd->newmedia) {					    
      if (!(sd->pe->sink2 = element_create ("sink2", "multifilesink"))) return 1;
      /* attach signal */
      g_signal_connect (G_OBJECT(sd->pe->sink2), "newfile", (GCallback)newfile_signal, sd);
    }
    else {
      if (!(sd->pe->sink2 = element_create ("sink2", "filesink"))) return 1;
    }
    g_object_set (G_OBJECT (sd->pe->sink2), "location", sd->webcaststream_fname, NULL);
    sd->pe->tee2_src1 = gst_element_get_request_pad(sd->pe->tee2, "src%d");
    gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->sink2);
    gst_pad_connect (sd->pe->tee2_src1, gst_element_get_pad (sd->pe->sink2, "sink"));
    sd->sub_stream_status[1]=1;
    return 0;
  }
  return -1;
}

int add_webcast_substream(struct stream_details* sd,gboolean start)
{

  if (!sd->sub_stream_status[2]) {
    g_print ("Add & Connect Webcast Stream 3 Server Type: %s IP:%s Port: %d Mount: %s\n", sd->webcastserver,sd->iceip,sd->iceport,sd->icemount);
    if (!start) {
      remove_webcast_encoder(sd);
      add_webcast_encoder(sd,FALSE);
    }
    if (ACAST_HAVE_SHOUT2(sd)) {
      if (g_strcasecmp(sd->webcasttype,"ogg")==0 || g_strcasecmp(sd->webcastserver,"icecast2")==0) {
	if (!(sd->pe->sinkweb = element_create ("sinkweb", "shout2send"))) return 1;
	g_object_set(G_OBJECT(sd->pe->sinkweb), "protocol",3,NULL);
      }
    }

    if (g_strcasecmp(sd->webcasttype,"mp3")==0) {

      if (ACAST_HAVE_SHOUT2(sd)) {
	if (g_strcasecmp(sd->webcastserver,"icecast2")==0) {
	  if (!(sd->pe->sinkweb = element_create ("sinkweb", "shout2send"))) return 1;
	  g_object_set(G_OBJECT(sd->pe->sinkweb), "protocol",3,NULL);
	  /*g_object_set(G_OBJECT(sd->pe->sinkweb), "sync", TRUE, NULL);*/
	}
	else if (g_strcasecmp(sd->webcastserver,"icecast1")==0) {
	  if (!(sd->pe->sinkweb = element_create ("sinkweb", "shout2send"))) return 1;
	  g_object_set(G_OBJECT(sd->pe->sinkweb), "protocol",1,NULL);
	}
	else if (g_strcasecmp(sd->webcastserver,"shoutcast")==0) {
	  if (!(sd->pe->sinkweb = element_create ("sinkweb", "shout2send"))) return 1;
	  g_object_set(G_OBJECT(sd->pe->sinkweb), "protocol",2,NULL);
	}
      }
      else if (ACAST_HAVE_SHOUT1(sd)) {
	if (g_strcasecmp(sd->webcastserver,"icecast1")==0) {
	  if (!(sd->pe->sinkweb = element_create ("sinkweb", "icecastsend"))) return 1;
	  g_object_set(G_OBJECT(sd->pe->sinkweb), "icy",FALSE,NULL);
	}
	else if (g_strcasecmp(sd->webcastserver,"shoutcast")==0) {
	  if (!(sd->pe->sinkweb = element_create ("sinkweb", "icecastsend"))) return 1;
	  g_object_set(G_OBJECT(sd->pe->sinkweb), "icy",TRUE,NULL);
	}
      }
    }
    
    g_object_set (G_OBJECT(sd->pe->sinkweb), "ip", sd->iceip, NULL);
    g_object_set (G_OBJECT(sd->pe->sinkweb), "port", sd->iceport, NULL);
    g_object_set (G_OBJECT(sd->pe->sinkweb), "mount", sd->icemount, NULL);
    g_object_set (G_OBJECT(sd->pe->sinkweb), "password", sd->icepass, NULL);
    
    g_print ("Add & Connect Webcast Stream\n");
    sd->pe->tee2_src2 = gst_element_get_request_pad(sd->pe->tee2, "src%d");
    gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->sinkweb);
    gst_pad_connect (sd->pe->tee2_src2, gst_element_get_pad (sd->pe->sinkweb, "sink"));
    sd->sub_stream_status[2]=1;
    return 0;
  }
  return -1;
}

int add_visualisation_substream(struct stream_details* sd)
{
	
  if (sd->gui) {
    g_print ("Add & Connect Visualisation Stream\n");
#ifdef GST_06
    if (!(sd->pe->s2m = element_create("s2m","stereo2mono"))) return 1;
    if (!(sd->pe->vis = element_create("vis","monoscope"))) return 1;
    
#else
    if (!(sd->pe->s2m = element_create("s2m","audioconvert"))) return 1;
    
    if (!(sd->pe->vis = element_create("vis",sd->visplugin))) return 1;
    
#endif
    if (!(sd->pe->queue = element_create("queue1","queue"))) return 1;
#ifdef GST_06
    if (!(sd->pe->color = element_create("color","colorspace"))) return 1;
    if (!(sd->pe->vsink = element_create("vsink","xvideosink"))) return 1;
#else
    if (!(sd->pe->color = element_create("color","ffmpegcolorspace"))) return 1;
    
    if (!(sd->pe->vsink = element_create("vsink","ximagesink"))) return 1;
    //g_object_set(G_OBJECT(sd->pe->vsink),"synchronous",TRUE,NULL);
    //gst_x_overlay_set_xwindow_id(GST_X_OVERLAY(sd->pe->vsink),0);
#endif
    
    sd->pe->tee1_src3 = gst_element_get_request_pad(sd->pe->tee,"src%d");
    sd->pe->vis_thread = gst_thread_new(g_strdup_printf("visthread%d",g_random_int())); // UGLY HACK gst problem

    gst_bin_add_many (GST_BIN (sd->pe->vis_thread), sd->pe->queue, sd->pe->s2m, sd->pe->vis,
		      sd->pe->color, sd->pe->vsink, NULL);
    
    gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->vis_thread);
    gst_pad_connect (sd->pe->tee1_src3, 
		     gst_element_get_pad (sd->pe->queue,"sink"));
    gst_element_link_many (sd->pe->queue, sd->pe->s2m, sd->pe->vis, 
			   sd->pe->color, sd->pe->vsink, NULL);
    sd->sub_stream_status[3]=1;
#ifndef GST_06
    g_signal_connect(GST_X_OVERLAY(sd->pe->vsink),"desired_size_changed",G_CALLBACK(xvimagesink_have_size),sd->gsgui);

    if (sd->gsgui->realised) embed_xvideosink(sd->gsgui);
#endif
    return 0;
  }
  
  return -1;
}

int add_listen_substream(struct stream_details* sd)
{
  /* FIXME: allow specification of audio sink for listen */
  if (!sd->sub_stream_status[5]) {
    g_print ("Add and connect listen Stream\n");
    sd->pe->listen_thread = gst_thread_new("listen_thread");
    if (!(sd->pe->listen_queue = element_create("listen_queue","queue"))) return 1;
#ifdef GNOME
    sd->pe->audiosink = gst_gconf_get_default_audio_sink();
#else
    if (!(sd->pe->audiosink = element_create("audiosink","osssink"))) return 1;
#endif
    sd->pe->tee1_src5 = gst_element_get_request_pad (sd->pe->tee, "src%d");
    gst_bin_add_many (GST_BIN(sd->pe->listen_thread), sd->pe->listen_queue,
		      sd->pe->audiosink, NULL);
    gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->listen_thread);

    gst_pad_connect (sd->pe->tee1_src5, 
		     gst_element_get_pad(sd->pe->listen_queue,"sink"));
    gst_pad_connect (gst_element_get_pad(sd->pe->listen_queue,"src"),
		     gst_element_get_pad (sd->pe->audiosink, "sink"));
    sd->sub_stream_status[5]=1;
    return 0;
  }
  return -1;
}

int add_wav_substream1(struct stream_details* sd)
{

  if (!sd->sub_stream_status[4]) {
    g_print ("Add & Connect .wav Encoding and Recording Stream\n");
    sd->curcounter++;
    sd->wavstream_fname=g_strdup_printf("%s/%s_%s_%d.wav",sd->dirname, sd->str,
					sd->datestr, sd->curcounter);
    if (!(sd->pe->enc3 = element_create ("enc3","wavenc"))) return 1;
    if (!(sd->pe->sink3 = element_create ("sink3", "filesink"))) return 1;
    g_object_set (G_OBJECT (sd->pe->sink3), "location", sd->wavstream_fname, NULL);
    /* request one pad from tee */
    sd->pe->tee1_src4 = gst_element_get_request_pad (sd->pe->tee, "src%d");
    gst_bin_add_many (GST_BIN (sd->pe->pipeline), sd->pe->enc3, sd->pe->sink3, NULL);
    gst_pad_connect (sd->pe->tee1_src4, gst_element_get_pad (sd->pe->enc3, "sink"));
    gst_pad_connect (gst_element_get_pad(sd->pe->enc3,"src"), 
		     gst_element_get_pad(sd->pe->sink3,"sink"));
    sd->sub_stream_status[4]=1;
    return 0;
  }
  return -1;
}

int switch_file(struct stream_details* sd)
{
  GstElement *tsink1, *tsink2, *tsink3;
  
  if (sd->newmedia) {
    gst_newmedia_trigger (GST_NEWMEDIA(sd->pe->newmedia));
    return 0;
  }
  if (sd->sub_stream_status[0]) {
    if (!(tsink1 = element_create ("sink1", "filesink"))) return 1;
    g_free(sd->recordstream_fname);
  }
  if (sd->sub_stream_status[1]) {
    if (!(tsink2 = element_create ("sink2","filesink"))) return 1;
    g_free(sd->webcaststream_fname);
  }
  
  if (sd->sub_stream_status[4]) {
    if (!(tsink3 = element_create ("sink3","filesink"))) return 1;
    g_free(sd->wavstream_fname);
  }
  
  sd->curcounter++;
  
  if (sd->sub_stream_status[1]) {
    sd->webcaststream_fname=g_strdup_printf("%s/%s_%s_1_%d.%s", sd->dirname,
					    sd->str,sd->datestr,sd->curcounter,
					    g_strcasecmp(sd->webcasttype,"ogg")==0 ? "ogg" : "mp3");
    printf("New Broadcast Filename %s \n",sd->webcaststream_fname);
  }
  
  if (sd->sub_stream_status[4]) {
    sd->wavstream_fname=g_strdup_printf("%s/%s_%s_4_%d.wav", sd->dirname,
					sd->str,sd->datestr,sd->curcounter);
    printf("New Broadcast Filename %s \n",sd->wavstream_fname);
  }
  
  if (sd->sub_stream_status[1]) {
    g_object_set(G_OBJECT(tsink2),"location",sd->webcaststream_fname,NULL);
  }
  if (sd->sub_stream_status[4]) {
    g_object_set(G_OBJECT(tsink3),"location",sd->wavstream_fname,NULL);
  }
  
  if (sd->sub_stream_status[0]) {
    stop_record_stream(sd);
    add_record_stream(sd);
  }
  
  if (sd->sub_stream_status[1]) {
    remove_webcast_encoder(sd);
    add_webcast_encoder(sd,FALSE);
    
    gst_pad_disconnect (sd->pe->tee2_src1,
			gst_element_get_pad(sd->pe->sink2,"sink"));
    gst_bin_remove(GST_BIN(sd->pe->pipeline),sd->pe->sink2);
    gst_bin_add(GST_BIN(sd->pe->pipeline),tsink2);
    gst_pad_connect (sd->pe->tee2_src1,
		     gst_element_get_pad(tsink2,"sink"));
    // free sink2
    //gst_element_destroy(sink2);
    sd->pe->sink2=tsink2;
  }

  if (sd->sub_stream_status[4]) {
    gst_pad_disconnect (gst_element_get_pad(sd->pe->enc3,"src"),
			gst_element_get_pad(sd->pe->sink3,"sink"));
    gst_pad_disconnect (sd->pe->tee1_src4,
			gst_element_get_pad(sd->pe->enc3,"sink"));
    gst_bin_remove_many (GST_BIN(sd->pe->pipeline),sd->pe->sink3, sd->pe->enc3, NULL);
    
    gst_element_release_request_pad(sd->pe->tee, sd->pe->tee1_src4);
		
    if (!(sd->pe->enc3 = element_create ("enc3","wavenc"))) return -1;
    
    /* request one pad from tee */
    sd->pe->tee1_src4 = gst_element_get_request_pad (sd->pe->tee, "src%d");
    gst_bin_add_many (GST_BIN (sd->pe->pipeline), sd->pe->enc3, tsink3, NULL);
    gst_pad_connect (sd->pe->tee1_src4, gst_element_get_pad (sd->pe->enc3, "sink"));
    gst_pad_connect(gst_element_get_pad(sd->pe->enc3,"src"), 
		    gst_element_get_pad(tsink3,"sink"));
    sd->pe->sink3=tsink3;
  }
  
  
}

gboolean
continue_pipeline(gpointer data)
{
  struct stream_details* sd=(struct stream_details*)data;
  if (sd->gui == FALSE || sd->active == 1) {
  	if (gst_bin_iterate(GST_BIN(sd->pe->pipeline))) return TRUE;
  }
  else {
  	return TRUE;
  }
  //if (sd->gui) gtk_main_quit();
  if (sd->gui) {
  	sd->active=0;
  	// pipeline stopped so remove
  	g_idle_remove_by_data(data);
  	return TRUE;	
  }
  else g_main_loop_quit(sd->mainloop);
  return FALSE;
}

void acast_start_gst_pipeline (struct stream_details* sd)
{
  /* set to play */
  if (sd->gui == 0 || sd->active == 1) {
    g_print ("Playing Pipeline\n");
    display_stream_status(sd);
    gst_element_set_state (sd->pe->pipeline, GST_STATE_PLAYING);
    g_idle_add (continue_pipeline, sd);
  }
}

void acast_destroy_gst_pipeline (struct stream_details* sd)
{
  if (sd->pe != NULL) {
    gst_element_set_state (sd->pe->pipeline, GST_STATE_NULL);
    
    g_free (sd->pe);
  
    sd->pe = NULL;
  }
}

int acast_setup_gst_pipeline (struct stream_details* sd)
{
  sd->pe = g_new0(struct pipeline_elements, 1);

  /* create the initial pipeline */
  sd->pe->pipeline = gst_pipeline_new("pipeline");

  /* create the input elements */
  if (!(sd->pe->tee = element_create ("tee", "tee"))) return 1;
#ifdef GNOME
  if (g_strcasecmp (sd->audiosrc, "gnome") == 0) {
    sd->pe->src =  gst_gconf_get_default_audio_src ();
  }
  else
#endif
    if (!(sd->pe->src = element_create ("src", sd->audiosrc))) return 1;
  if (g_strcasecmp(sd->audiosrc,"filesrc")==0) {
    g_object_set(G_OBJECT(sd->pe->src), "location", sd->audiodevice, NULL);
    if (!(sd->pe->filedec = element_create("filedec","spider"))) return 1;
  }
  else if (g_strcasecmp(sd->audiosrc,"esdmon")!=0 && g_strcasecmp(sd->audiosrc,"gnome")!=0) { /* if osssrc or alsasrc */
    g_object_set (G_OBJECT(sd->pe->src),"device",sd->audiodevice, NULL);
  }
#ifndef GST_06
  sd->pe->newmedia = GST_ELEMENT(g_object_new(GST_TYPE_NEWMEDIA, NULL));
  gst_element_set_name (sd->pe->newmedia, "newmedia");
#endif
  if (!(sd->pe->level = element_create("level","level"))) return 1;
  g_object_set (G_OBJECT(sd->pe->level),"interval",0.05,NULL);
  g_object_set (sd->pe->level, "signal", TRUE, NULL);
  g_signal_connect (sd->pe->level, "level", G_CALLBACK (level_callback), sd->gsgui);
  
  /* add the input elements to the pipeline */
  gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->src);
  if (g_strcasecmp(sd->audiosrc,"filesrc")==0) {
    gst_bin_add (GST_BIN (sd->pe->pipeline), sd->pe->filedec);
  }
#ifndef GST_06
  gst_bin_add (GST_BIN(sd->pe->pipeline), sd->pe->newmedia);
#endif
  gst_bin_add_many (GST_BIN (sd->pe->pipeline), sd->pe->level, sd->pe->tee, NULL);

  /* link input elements */
  if (g_strcasecmp(sd->audiosrc,"filesrc")==0) {
  	gst_element_link (sd->pe->src, sd->pe->filedec);
#ifndef GST_06
	gst_pad_connect(gst_element_get_pad (sd->pe->filedec, "src%d"),
			gst_element_get_pad (sd->pe->newmedia, "sink"));
	gst_pad_connect(gst_element_get_pad (sd->pe->newmedia, "src"),
			gst_element_get_pad (sd->pe->level,"sink"));
#else
	gst_pad_connect(gst_element_get_pad (sd->pe->filedec, "src%d"),
			gst_element_get_pad (sd->pe->level,"sink"));
#endif
  }
  else {
    GstCaps* tempcaps = gst_caps_from_string ("audio/x-raw-int, rate=44100, channels=2, signed=(boolean)true, width=16, depth=16");
#ifndef GST_06
    gst_pad_link_filtered(gst_element_get_pad (sd->pe->src, "src"),
			  gst_element_get_pad (sd->pe->newmedia, "sink"),
			  tempcaps);
    gst_pad_connect(gst_element_get_pad (sd->pe->newmedia, "src"),
		    gst_element_get_pad (sd->pe->level,"sink"));
#else
    gst_pad_link_filtered(gst_element_get_pad (sd->pe->src, "src"),
			  gst_element_get_pad (sd->pe->level,"sink"), tempcaps);
#endif
    gst_caps_free(tempcaps);
  }
  gst_element_link (sd->pe->level, sd->pe->tee);

  /* add the recording and webcast streams */
  if (sd->sub_stream_status[0]==1) {
    sd->sub_stream_status[0]=0;
    add_record_stream(sd); 
  }
	
  add_webcast_encoder(sd,TRUE);
  
  if (sd->sub_stream_status[1]==1) {
    sd->sub_stream_status[1]=0;
    
    add_webcast_record_stream(sd,TRUE);
  }
  if (sd->sub_stream_status[2]==1) {
    sd->sub_stream_status[2]=0;
    add_webcast_substream(sd,TRUE);
  }
  if (sd->sub_stream_status[3]==1) {
    sd->sub_stream_status[3]=0;
    add_visualisation_substream(sd);
  }
  
  if (sd->sub_stream_status[4]==1) {
    sd->sub_stream_status[4]=0;
    add_wav_substream1(sd);
  }
  return 0;
}

int
acast_main (struct stream_details* sd)
{
  struct timeval tv;
  fd_set rfds;
  int fd;
  int fdr;
  char ch;
  GIOChannel* stdin_ioc;
  time_t ct;
  size_t n;
  int inpcmd;
  int i;
  struct gstsave_gui gsgui;

  gsgui.sd=sd;
  gsgui.realised=FALSE;
  sd->gsgui=&gsgui;
  if (g_file_test ("blah", G_FILE_TEST_EXISTS)) {
    fdr=open("blah",O_RDONLY|O_NONBLOCK,NULL);
    fd=open("blah",O_WRONLY|O_NONBLOCK,NULL);
    
    sd->ioc=g_io_channel_unix_new(fd);
  }
  else {
    sd->ioc = NULL;
  }
  /* Create Filename suffix based on Date */
  ct = time(&ct);
  // strftime(datestr, 30, "%Y%m%d%a", localtime(&ct));
  strftime(sd->datestr, 30, "%Y%m%d", localtime(&ct));
  
  /* create */
  tv.tv_sec = 0;		/* Timeout in sec. */
  tv.tv_usec = 0;		/* Timeout in microseconds */
  
  sd->recordstream_fname=g_strdup_printf("%s/%s_%s_%d_%d.%s",sd->dirname, sd->str, sd->datestr,
				  sd->bitrate[0],sd->curcounter,
				  g_strcasecmp(sd->recordtype,"ogg")==0 ? "ogg" : "mp3");
  sd->webcaststream_fname=g_strdup_printf("%s/%s_%s_%d_%d.%s", sd->dirname, sd->str, sd->datestr,
				  sd->bitrate[1], sd->curcounter,
				  g_strcasecmp(sd->webcasttype,"ogg")==0 ? "ogg" : "mp3");
  sd->wavstream_fname=g_strdup_printf("%s/%s_%s_%d.wav", sd->dirname, sd->str, sd->datestr,
				  sd->curcounter);

  if (sd->gui == 0 || sd->active == 1) { 
    if (acast_setup_gst_pipeline (sd) == 1) return 1;
    acast_start_gst_pipeline (sd);
  }

  stdin_ioc = g_io_channel_unix_new (0); // stdin fd = 0
  g_io_add_watch (stdin_ioc,G_IO_IN,keyboard_input,sd);
  if (sd->gui == 1) {
    setup_gui (&gsgui);
    sd->mainloop = NULL;
    gtk_main();
  }
  else {
    sd->mainloop=g_main_loop_new(NULL,FALSE);
    
    g_main_loop_run(sd->mainloop);
  }

  g_print ("Done !\n");
  if (sd->ioc) {
    close(fdr);
    close(fd);
  }
  return 0;
}

gboolean acast_is_element_available (GHashTable* available_elements, const gchar* element_name)
{
  return (g_hash_table_lookup(available_elements, element_name) != NULL);
}

void acast_add_available_element (GHashTable* available_elements, const gchar* element_name)
{
  if (element_name!=NULL && available_elements!=NULL) {
    GstElementFactory* ef;
    ef = gst_element_factory_find(element_name);
    if (ef != NULL) g_hash_table_insert (available_elements, g_strdup(element_name), ef);
  }
}
void acast_set_defaults(struct stream_details* sd)
{
  /* find available elements */
  /* audio sources  and sinks */
  acast_add_available_element(sd->available_elements, "osssrc");
  acast_add_available_element(sd->available_elements, "alsasrc");
  acast_add_available_element(sd->available_elements, "esdmon");
  acast_add_available_element(sd->available_elements, "osssink");
  acast_add_available_element(sd->available_elements, "alsasink");
  acast_add_available_element(sd->available_elements, "esdsink");
  acast_add_available_element(sd->available_elements, "osxaudiosrc");
  acast_add_available_element(sd->available_elements, "osxaudiosink");
  /* general elements */
  acast_add_available_element(sd->available_elements, "tee");
  acast_add_available_element(sd->available_elements, "queue");
  acast_add_available_element(sd->available_elements, "audioconvert");
  acast_add_available_element(sd->available_elements, "audioscale");
  /* encoders and muxers */
  acast_add_available_element(sd->available_elements, "lame");
  acast_add_available_element(sd->available_elements, "vorbisenc");
  acast_add_available_element(sd->available_elements, "rawvorbisenc");
  acast_add_available_element(sd->available_elements, "oggmux");
  acast_add_available_element(sd->available_elements, "wavenc");
  /* streaming server sinks */
  acast_add_available_element(sd->available_elements, "shout2send");
  acast_add_available_element(sd->available_elements, "icecastsend");
  acast_add_available_element(sd->available_elements, "tcpserversink");
  /* visualisation */
  acast_add_available_element(sd->available_elements, "goom");
  acast_add_available_element(sd->available_elements, "monoscope");
  acast_add_available_element(sd->available_elements, "ximagesink");
  acast_add_available_element(sd->available_elements, "xvimagesink");

  sd->config_fname = NULL;
  sd->dirname = g_strdup(".");
  sd->str = g_strdup("acastsave");
  sd->active = 0;
  sd->visplugin = NULL;
  /* visualisation */
  if (acast_is_element_available (sd->available_elements, "ximagesink") || 
      acast_is_element_available (sd->available_elements, "xvimagesink")) {
    if (acast_is_element_available (sd->available_elements, "goom")) {
      sd->visplugin = g_strdup("goom");
      sd->sub_stream_status[3] = 0;
    }
    else if (acast_is_element_available (sd->available_elements, "monoscope")) {
      sd->visplugin = g_strdup("monoscope");
      sd->sub_stream_status[3] = 0;
    }
    else {
      sd->sub_stream_status[3] = -1;
    }
  }
  else {
    sd->sub_stream_status[3] = -1;
  }
  
  /* record stream */
  if (acast_is_element_available (sd->available_elements, "rawvorbisenc")) {
    sd->recordtype = g_strdup("ogg");
    sd->bitrate[0] = 128;
    sd->quality[0] = 0;
    sd->mode[0] = 3;
    sd->sub_stream_status[3] = 0;
  }
  else if (acast_is_element_available (sd->available_elements, "lame")) {
    sd->recordtype = g_strdup("mp3");
    sd->bitrate[0] = 128;
    sd->quality[0] = 0;
    sd->mode[0] = 3;
    sd->sub_stream_status[3] = 0;
  }
  else {
    g_critical("Please install either the vorbis or lame plugin from gst-plugins");
    exit(-1);
  }
  /* webcast stream */
  if (ACAST_HAVE_SHOUT2(sd)) {  
    sd->webcastserver = g_strdup("icecast2");
    if (ACAST_HAVE_VORBIS(sd)) {
      sd->webcasttype = g_strdup("ogg");
    }
    else {
      sd->webcasttype = g_strdup("mp3");
    }
  }
  else if (ACAST_HAVE_SHOUT1(sd)) {
    sd->webcastserver = g_strdup("icecast1");
    sd->webcasttype = g_strdup("mp3");
  }
  else {
    g_critical("Please install either the shout2 or shout plugin from gst-plugins");
    exit(-1);
  }
  sd->bitrate[1] = 16;
  sd->mode[1] = 3;
  sd->quality[1] = 0;
  sd->sub_stream_status[1] = 0;
  sd->sub_stream_status[2] = 0;
  sd->iceip = g_strdup("127.0.0.1");
  sd->iceport = 8000;
  sd->icemount = g_strdup("/acast");
  sd->icepass = g_strdup("hackme");
  /* wave stream */
  if (ACAST_HAVE_WAV(sd)) {  
    sd->sub_stream_status[4] = 0;
  }
  else {
    sd->sub_stream_status[4] = -1;
  }
  /* input */
  if (ACAST_HAVE_OSS(sd)) {  
    sd->audiosrc = g_strdup("osssrc");
    sd->audiodevice = g_strdup("/dev/dsp");
  }
  else if (ACAST_HAVE_ALSA(sd)) {  
    sd->audiosrc = g_strdup("alsasrc");
    sd->audiodevice = g_strdup("default");
  }
  else if (ACAST_HAVE_ESD(sd)) {
    sd->audiosrc = g_strdup("esdmon");
    sd->audiodevice = NULL;
  }
  else {
    g_critical("Please install either the oss, alsa or esd plugins from gst-plugins");
    exit(-1);
  }

  /* check for new media support */
  sd->newmedia = FALSE;

  {
    GstPlugin* multifilesink_plugin = gst_registry_pool_find_plugin ("multifilesink");
    if (multifilesink_plugin && 
	multifilesink_plugin->desc.version[2] == '8' &&
	atof(multifilesink_plugin->desc.version+4) > 4.1) {
      sd->newmedia = TRUE;
      
    }
  }
    
}

void acast_init(struct stream_details* sd)
{
  int i;
  
  sd->audiodevice=NULL;
  sd->audiosrc=NULL;
  sd->iceip=NULL;
  sd->dirname=NULL;
  sd->icepass=NULL;
  sd->recordstream_fname=NULL;
  sd->webcaststream_fname=NULL;
  sd->wavstream_fname=NULL;
  sd->str=NULL;
  sd->webcastserver=NULL;
  sd->recordtype=NULL;
  sd->webcasttype=NULL;
  sd->config_fname=NULL;
  sd->curcounter=0;
  sd->consolelevel=FALSE;
  sd->relays = g_hash_table_new(g_str_hash, g_str_equal);
  sd->available_elements = g_hash_table_new(g_str_hash, g_str_equal);
  for(i=0;i<6;i++) {
    sd->sub_stream_status[i]=0;
  }
  for(i=0;i<2;i++) {
    sd->quality[i]=0;
  }
  sd->active=1;
  sd->newmedia = FALSE;
}



int main(int argc, char* argv[])
{
  struct stream_details sd;
  char* config_filename = NULL;
  char* base_filename = NULL;
  gboolean console = FALSE;
  struct poptOption options[] = {
    {"configfile", 'f', POPT_ARG_STRING | POPT_ARGFLAG_STRIP, &config_filename, 0,
     "specify config filename", NULL},
    {"basefile", 'b', POPT_ARG_STRING | POPT_ARGFLAG_STRIP, &base_filename, 0,
     "override base filename with", NULL},
    {"console", 'c', POPT_ARG_NONE | POPT_ARGFLAG_STRIP, &console, 0,
     "use console mode", NULL},
    POPT_TABLEEND
   };
#ifdef GNOME
  GConfClient *client;
#endif
  /* init */
  
  acast_init(&sd);
  	  	
  gst_init_with_popt_table(&argc,&argv, options);  
  
  if (console || getenv("DISPLAY") == NULL) {

    sd.gui = 0;
  }
  else {
    sd.gui = 1;
#if 0
    g_thread_init (NULL);
    gdk_threads_init ();
#endif
#ifdef GNOME
    program = gnome_program_init ("acast",
				  "0.17",
				  LIBGNOMEUI_MODULE,
				  argc, argv,
				  GNOME_PROGRAM_STANDARD_PROPERTIES,
				  GNOME_PARAM_HUMAN_READABLE_NAME, "acast audio broadcaster",
				  GNOME_PARAM_APP_DATADIR, ACAST_DATADIR,
				  NULL);
    client = gconf_client_get_default ();
    gnome_media_profiles_init (client);
    g_object_unref (G_OBJECT (client));
    sd.recording_gm_profiles = gm_audio_profile_get_active_list();
#else
    gtk_init(&argc,&argv);
#endif
  }    
 
  acast_set_defaults (&sd); 
  if (config_filename!=NULL) {
    sd.config_fname = config_filename;
    acast_parse_configfile (&sd);
  }
  if (base_filename!=NULL) {
    if (sd.str) g_free (sd.str);
    sd.str = base_filename;
  }
  return acast_main (&sd);
 
}
